#    Massive Enumeration Toolset
#    Copyright (C) 2005  Petko Petkov (GNUCITIZEN) ppetkov@gnucitizen.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import re
import base64
import urllib2
import xml.dom.minidom

class GoogleApiTool:
	def makeSOAPmessage(self, method, parameters, key):
		if len(key) != 32:
			raise ValueError, 'invalid license key provided'

		message = \
		'''<?xml version="1.0" encoding="UTF-8"?>
		<SOAP-ENV:Envelope 
		xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
		xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" 
		xmlns:xsd="http://www.w3.org/1999/XMLSchema">

		<SOAP-ENV:Body>
		<ns1:%(method)s xmlns:ns1="urn:GoogleSearch" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
		<key xsi:type="xsd:string">%(key)s</key>%(parameters)s
		</ns1:%(method)s>
		</SOAP-ENV:Body>
		</SOAP-ENV:Envelope>'''

		msgparameters = ''

		for name, value, type in parameters:
			if type == 'boolean':
				if value == True:
					value = 'true'

				else:
					value = 'false'

			msgparameters += '\n<%s xsi:type="xsd:%s">%s</%s>' % (name, type, value, name)

		return message % {'method':method, 'parameters':msgparameters, 'key':key}

	def parseSOAPresponse(self, message):
		return message

	def sendSOAPrequest(self, method, parameters, key):
		message = self.makeSOAPmessage(method, parameters, key)

		try:
			request = urllib2.Request(url = 'http://api.google.com/search/beta2', data = message)
			request.add_header('Content-Type', 'text/xml; charset="UTF-8"')
			request.add_header('SOAPAction', 'urn:GoogleSearchAction')

			opener = urllib2.build_opener()
			response = opener.open(request).read()

		except:
			raise RuntimeError, 'unable to request from api.google.com'

		return self.parseSOAPresponse(response)

class GoogleWebTool(GoogleApiTool):
	def __init__(self, key = None):
		self.key = key

		self.__dstats = \
		{'startIndex': None, 
		'endIndex': None, 
		'estimatedTotal': None, 
		'estimateIsExact': None, 
		'searchTime': None,
		'searchTips': None, 
		'searchComments': None}

		self.__stats = None
		self.__lstats = None
		self.__lparams = None

	def parseSOAPresponse(self, message):
		try:
			xmldoc = xml.dom.minidom.parseString(message)

		except:
			raise RuntimeError, 'unfamiliar data stucture'

		stats = {}
		records = []

		for item in xmldoc.getElementsByTagName('item'):
			record = {}

			record['URL'] = item.getElementsByTagName('URL')[0].firstChild
			record['cachedSize'] = item.getElementsByTagName('cachedSize')[0].firstChild
			record['snippet'] = item.getElementsByTagName('snippet')[0].firstChild
			record['summary'] = item.getElementsByTagName('summary')[0].firstChild
			record['title'] = item.getElementsByTagName('title')[0].firstChild

			for key, value in record.iteritems():
				if value != None:
					record[key] = value.data

			records.append(record)

		if len(records) == 0:
			return None, None

		else:
			stats['startIndex'] = int(xmldoc.getElementsByTagName('startIndex')[0].firstChild.data)
			stats['endIndex'] = int(xmldoc.getElementsByTagName('endIndex')[0].firstChild.data)
			stats['estimatedTotal'] = int(xmldoc.getElementsByTagName('estimatedTotalResultsCount')[0].firstChild.data)
			stats['estimateIsExact'] = xmldoc.getElementsByTagName('estimateIsExact')[0].firstChild.data
			stats['searchTime'] = float(xmldoc.getElementsByTagName('searchTime')[0].firstChild.data)
			stats['searchTips'] =  xmldoc.getElementsByTagName('searchTips')[0].firstChild
			stats['searchComments'] =  xmldoc.getElementsByTagName('searchComments')[0].firstChild

			if stats['searchTips'] != None:
				stats['searchTips'] = stats['searchTips'].data

			if stats['searchComments'] != None:
				stats['searchComments'] = stats['searchComments'].data

			if stats['estimateIsExact'] == 'true':
				stats['estimateIsExact'] = True

			else:
				stats['estimateIsExact'] = False

		return stats, records

	def stats(self):
		return self.__stats

	def search(self, query, start = 0, maxresults = 10, filter = True, restrict = '', safesearch = False, lr = 'en', ie = 'utf-8', oe = 'utf-8', key = None):
		if key == None:
			if self.key == None:
				raise ValueError, 'no license key provided'

			else:
				key = self.key

		parameters = \
		[('q', query, 'string'),
		('start', start, 'int'), 
		('maxResults', maxresults, 'int'), 
		('filter', filter, 'boolean'),
		('restrict', restrict, 'string'),
		('safeSearch', safesearch, 'boolean'),
		('lr', lr, 'string'),
		('ie', ie, 'string'),
		('oe', oe, 'string')]

		stats, records = self.sendSOAPrequest('doGoogleSearch', parameters, key)

		if stats != None:
			self.__stats = self.__dstats
			self.__stats.update(stats)

		if self.__lparams != None and self.__lstats != None:
			if self.__lparams == repr(parameters) and self.__lstats == repr(stats):
				return None

		self.__lstats = repr(stats)
		self.__lparams = repr(stats)

		return records

class GoogleSpellTool(GoogleApiTool):
	def __init__(self, key = None):
		self.key = key
		self.__phrase = None

	def parseSOAPresponse(self, message):
		try:
			xmldoc = xml.dom.minidom.parseString(message)
			ereturn = xmldoc.getElementsByTagName('return')[0].firstChild

		except:
			raise RuntimeError, 'unfamiliar data stucture'

		if ereturn == None:
			return self.__phrase

		return ereturn.data

	def search(self, phrase, key = None):
		if key == None:
			if self.key == None:
				raise ValueError, 'no license key provided'

			else:
				key = self.key

		self.__phrase = phrase

		return self.sendSOAPrequest('doSpellingSuggestion', [('phrase', phrase, 'string')], key)

class GoogleCacheTool(GoogleApiTool):
	def __init__(self, key = None):
		self.key = key

		self.__stats = None

	def parseSOAPresponse(self, message):
		try:
			xmldoc = xml.dom.minidom.parseString(message)

		except:
			raise RuntimeError, 'unfamiliar data stucture'

		response = base64.b64decode(xmldoc.getElementsByTagName('return')[0].firstChild.data)

		if response.find('<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1">') == 0:
			tokens = response.split('<hr>')

			match = re.search('as retrieved on (.*)\.<br>', tokens[0])

			if match == None:
				return None, None

			return {'retrievedDate': match.group(1)}, ''.join(tokens[1:]).strip()

		else:
			return None, None

	def search(self, url, key = None):
		if key == None:
			if self.key == None:
				raise ValueError, 'no license key provided'

			else:
				key = self.key

		stats, result = self.sendSOAPrequest('doGetCachedPage', [('url', url, 'string')], key)

		self.__stats = stats

		return result
