#    Massive Enumeration Toolset
#    Copyright (C) 2005  Petko Petkov (GNUCITIZEN) ppetkov@gnucitizen.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import re
import string
import urllib
import urllib2
import xml.dom.minidom

class GoogleMobileTool:
	def extracturl(self, url):
		match = re.search('/u=(.*)/c=0', url)

		if match != None:
			url = match.group(1)

		match = re.match('.*://.*', url)

		if match == None:
			return 'http://' + url

		else:
			return url

	def decodeurl(self, url):
		return re.sub('@[\d\w]{2}', lambda t: chr(string.atoi(t.group(0)[1:], 16)), url)

	def fixgooglebug(self, data):
		return re.sub('&(\w*)\.\.\.', r'&amp;\1...', data)

	def makeHTTPrequest(self, type, parameters):
		return 'http://www.google.com/%s?%s' % (type, urllib.urlencode(parameters))

	def parseHTTPresponse(self, response):
		return response

	def sendHTTPrequest(self, type, parameters):
		request = self.makeHTTPrequest(type, parameters)

		try:
			request = urllib2.Request(request)
			request.add_header('User-Agent', 'Mozilla/5.0 Gecko/20050716 Firefox/1.0.6')

			opener = urllib2.build_opener()
			response = opener.open(request).read()

		except:
			raise RuntimeError, 'unable to request from www.google.com'

		return self.parseHTTPresponse(self.fixgooglebug(response))

class GoogleWebTool(GoogleMobileTool):
	def __init__(self):
		self.__dstats = \
		{'startIndex': None, 
		'endIndex': None, 
		'estimatedTotal': None, 
		'estimateIsExact': None, 
		'searchTime': None,
		'searchTips': None,
		'searchComments': None}

		self.__stats = None
		self.__lstats = None 
		self.__lparams = None

	def parseHTTPresponse(self, response):
		try:
			xmldoc = xml.dom.minidom.parseString(response)

		except:
			raise RuntimeError, 'unfamiliar data stucture'

		stats = {}
		records = []

		for a in xmldoc.getElementsByTagName('a'):
			if a.attributes.has_key('accesskey'):
				record = {}

				record['cachedSize'] = record['summary'] = None
				record['URL'] = self.extracturl(self.decodeurl(a.attributes['href'].value))
				record['title'] = a.firstChild and a.firstChild.data
				record['snippet'] = a.nextSibling and a.nextSibling.data.strip().strip('-').strip()

				records.append(record)

		if len(records) == 0:
			return None, None

		else:
			for div in xmldoc.getElementsByTagName('div')[1:3]:
				result = re.sub('</?[^>]*>', '', div.toxml())

				spellmatch = re.match('Did you mean:(.*)', result)
				exacresmatch = re.match('Web results:.*Results (\d+) - (\d+) of ([\d,]+).', result)
				resmatch = re.match('Web results:.*Results (\d+) - (\d+) of about ([\d,]+).', result)

				if spellmatch != None:
					stats['searchTips'] = spellmatch.group(1)

				if exacresmatch != None:
					stats['startIndex'] = int(exacresmatch.group(1))
					stats['endIndex'] = int(exacresmatch.group(2))
					stats['estimatedTotal'] = int(exacresmatch.group(3).replace(',', ''))
					stats['estimateIsExact'] = True

				if resmatch != None:
					stats['startIndex'] = int(resmatch.group(1))
					stats['endIndex'] = int(resmatch.group(2))
					stats['estimatedTotal'] = int(resmatch.group(3).replace(',', ''))
					stats['estimateIsExact'] = False

		return stats, records

	def stats(self):
		return self.__stats

	def search(self, query, start = 0, maxresults = 10, filter = True, restrict = '', safesearch = False, lr = 'en', ie = 'utf-8', oe = 'utf-8'):
		if filter == True:
			filter = 1

		else:
			filter = 0

		if safesearch == False:
			safesearch = 'off'

		else:
			safesearch = 'on'

		parameters = \
		{'q': query,
		'start': start,
		'num': maxresults,
		'filter': filter,
		'restrict': restrict,
		'safe': safesearch,
		'lr': lr,
		'ie': ie,
		'oe': oe}

		stats, records = self.sendHTTPrequest('xhtml', parameters)

		if stats != None:
			self.__stats = self.__dstats
			self.__stats.update(stats)

		if self.__lparams != None and self.__lstats != None:
			if self.__lparams == repr(parameters) and self.__lstats == repr(stats):
				return None

		self.__lstats = repr(stats)
		self.__lparams = repr(stats)

		return records

class GoogleImageTool(GoogleWebTool):
	def extractsrc(self, src):
		tokens = src.split(':')

		if len(tokens) == 3:
			src = tokens[2]

		match = re.match('.*://.*', src)

		if match == None:
			return 'http://' + src

		else:
			return src

	def parseHTTPresponse(self, response):
		try:
			xmldoc = xml.dom.minidom.parseString(response)

		except:
			raise RuntimeError, 'unfamiliar data stucture'

		stats = {}
		records = []

		as = xmldoc.getElementsByTagName('a')

		for i in range(0, len(as), 2):
			a = as[i]

			if a.attributes.has_key('accesskey'):
				record = {}

				record['page'] = self.extracturl(self.decodeurl(a.attributes['href'].value))
				record['URL'] = self.extractsrc(self.decodeurl(a.getElementsByTagName('img')[0].attributes['src'].value))

				records.append(record)

		if len(records) == 0:
			return None, None

		else:
			for div in xmldoc.getElementsByTagName('div')[1:3]:
				result = re.sub('</?[^>]*>', '', div.toxml())

				spellmatch = re.match('Did you mean:(.*)', result)
				exacresmatch = re.match('Image Results:.*Results (\d+) - (\d+) of ([\d,]+).', result)
				resmatch = re.match('Image Results:.*Results (\d+) - (\d+) of about ([\d,]+).', result)

				if spellmatch != None:
					stats['searchTips'] = spellmatch.group(1)

				if exacresmatch != None:
					stats['startIndex'] = int(exacresmatch.group(1))
					stats['endIndex'] = int(exacresmatch.group(2))
					stats['estimatedTotal'] = int(exacresmatch.group(3).replace(',', ''))
					stats['estimateIsExact'] = True

				if resmatch != None:
					stats['startIndex'] = int(resmatch.group(1))
					stats['endIndex'] = int(resmatch.group(2))
					stats['estimatedTotal'] = int(resmatch.group(3).replace(',', ''))
					stats['estimateIsExact'] = False

		return stats, records

	def sendHTTPrequest(self, type, parameters):
		parameters['site'] = 'images'
		return GoogleWebTool.sendHTTPrequest(self, type, parameters)

class GoogleSpellTool(GoogleMobileTool):
	def __ini__(self):
		self.__phrase = None

	def parseHTTPresponse(self, response):
		try:
			xmldoc = xml.dom.minidom.parseString(response)

		except:
			raise RuntimeError, 'unfamiliar data stucture'

		divs = xmldoc.getElementsByTagName('div')

		if len(divs) < 2:
			raise RuntimeError, 'google was unable to process your request'

		result = re.sub('</?[^>]*>', '', divs[1].toxml())

		spellmatch = re.match('Did you mean:(.*)', result)

		if spellmatch == None:
			return self.__phrase

		return spellmatch.group(1)

	def search(self, phrase):
		self.__phrase = phrase

		return self.sendHTTPrequest('xhtml', {'q': phrase})

class GoogleCacheTool(GoogleMobileTool):
	def __init__(self):
		self.__stats = None

	def parseHTTPresponse(self, response):
		if response.find('<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1">') == 0:
			tokens = response.split('<hr>')

			match = re.search('as retrieved on (.*)\.<br>', tokens[0])

			if match == None:
				return None, None

			return {'retrievedDate': match.group(1)}, ''.join(tokens[1:]).strip()

		else:
			return None, None

	def stats(self):
		return self.__stats

	def search(self, url):
		stats, result = self.sendHTTPrequest('search', {'q': 'cache:' + url})

		self.__stats = stats

		return result
